<?php
/**
 * Part of Linksys WRT3G tools
 *
 * PHP version 5
 *
 * @category Tools
 * @package  linksys-wrt3g-tools
 * @author   Christian Weiske <cweiske@cweiske.de>
 * @license  AGPL v3
 * @link     http://cweiske.de/linksys-wrt3g-tools.htm
 */
require_once 'HTTP/Request2.php';
require_once 'Wrt3g/HtmlParser.php';


/**
 * Main class to interact with the router.
 *
 * @category Tools
 * @package  linksys-wrt3g-tools
 * @author   Christian Weiske <cweiske@cweiske.de>
 * @license  AGPL v3
 * @link     http://cweiske.de/linksys-wrt3g-tools.htm
 */
class Wrt3g
{
    /**
     * Router hostname/IP
     *
     * @var string
     */
    public $host;

    /**
     * Name of user with administration privileges
     *
     * @var string
     */
    public $user;

    /**
     * Password for $user
     *
     * @var string
     */
    public $password;

    /**
     * Logging verbosity
     * 0 - no debug logging
     * 1 - show important details (connected URLs)
     * 2 - show internal details
     *
     * @var integer
     */
    public $verbosity = 0;



    /**
     * Returns the base URL to use for requests that require authentification.
     * Includes username, password and host.
     *
     * @return string URL without trailing slash after the host
     */
    protected function getAuthBaseUrl()
    {
        return 'http://'
            . $this->user
            . ':' . $this->password
            . '@' . $this->host;
    }



    /**
     * Returns the base URL to use for requests that do
     * not require authentification.
     *
     * @return string URL without trailing slash after the host
     */
    protected function getAnonBaseUrl()
    {
        return 'http://' . $this->host;
    }



    /**
    * Reboots the router.
    *
    * @return HTTP_Request2_Response
    *
    * @throws Exception When the router can't be reached, or unauthorized
    */
    public function reboot()
    {
        $url = $this->getAuthBaseUrl() . '/apply.cgi';
        $this->log('Connecting to ' . $url);

        $r = new HTTP_Request2();
        $r->setMethod(HTTP_Request2::METHOD_POST);
        $r->setUrl($url);
        $r->addPostParameter('action', 'Reboot');
        $r->addPostParameter('submit_button', 'Diagnostics');
        $r->addPostParameter('wait_time', 1);

        $resp = $r->send();
        return $resp;
    }//public function reboot()



    /**
    * Retrieves status information about the router
    *
    * @return array Array with several key-value pairs
    *               connection => connecting, disconnected, connected
    *
    * @throws Exception When the router can't be reached, or unauthorized
    */
    public function getStatus()
    {
        $arRetval = array();

        $strUrlBase = $this->getAuthBaseUrl();
        $parser = new Wrt3g_HtmlParser();

        /**
        * Connection status
        */
        $r = new HTTP_Request2();
        $r->setMethod(HTTP_Request2::METHOD_GET);
        $r->setUrl($strUrlBase . '/index_wstatus2.asp');
        $this->log('Connecting to ' . $strUrlBase . '/index_wstatus2.asp', 1);
        $resp = $r->send();
        $this->log($resp->getStatus() . ' ' . $resp->getReasonPhrase(), 1);
        $arRetval = $parser->index_wstatus2($resp->getBody());

        /**
        * GPRS/UMTS Status
        */
        $r->setUrl($strUrlBase . '/index_wstatus1.asp');
        $this->log('Connecting to ' . $strUrlBase . '/index_wstatus1.asp', 1);
        $resp = $r->send();
        $body = $resp->getBody();
        $arRetval = array_merge($arRetval, $parser->index_wstatus1($body));

        return $arRetval;
    }//public function getStatus()



    /**
     * Log a message to stdout.
     *
     * @param string  $msg   Message to display
     * @param integer $level Log level, see $verbosity
     *
     * @return void
     */
    protected function log($msg, $level = 1)
    {
        if ($this->verbosity >= $level) {
            echo $msg . "\n";
        }
    }
}
?>